@@ -3,7 +3,7 @@
  *  Visual Voicemail Daemon
  *
  *  Copyright (C) 2018 Purism SPC
- *                2020-2021 Chris Talbot <chris@talbothome.com>
+ *                2020-2022 Chris Talbot <chris@talbothome.com>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <gio/gio.h>
 #include <stdlib.h>
 #include <libmm-glib.h>
+#include "vvm3-activation.h"
 #include "vvm.h"
 #include "dbus.h"
 #include "vvmutil.h"
@@ -361,13 +362,13 @@ handle_method_call(GDBusConnection		*connection,
 						   "You have Visual Voicemail disabled");
 				}
 			} else {
-				vvm_error("You are not subscribed to the Visual Voicemail.");
+				g_critical("You are not subscribed to the Visual Voicemail.");
 				g_dbus_method_invocation_return_dbus_error(invocation,
 						   VVM_MODEMMANAGER_INTERFACE,
 						   "You are not subscribed to the Visual Voicemail.");
 			}
 		} else {
-			vvm_error("Modem is not ready to process any unsent/unreceived VVM messages.");
+			g_critical("Modem is not ready to process any unsent/unreceived VVM messages.");
 			g_dbus_method_invocation_return_dbus_error(invocation,
 				   VVM_MODEMMANAGER_INTERFACE,
 				   "Modem is not ready to process any unsent/unreceived VVM messages.");
@@ -416,10 +417,10 @@ mm_sync_vvm_imap_server(void)
 				DBG("You have Visual Voicemail disabled");
 			}
 		} else {
-			vvm_error("You are not subscribed to the Visual Voicemail.");
+			g_critical("You are not subscribed to the Visual Voicemail.");
 		}
 	} else {
-		vvm_error("Modem is not ready to process any unsent/unreceived VVM messages.");
+		g_critical("Modem is not ready to process any unsent/unreceived VVM messages.");
 	}
 }
 
@@ -450,7 +451,7 @@ vvm_process_status_message(struct sms_control_message *sms_msg)
 	g_autofree char *provisionstatus = NULL;
 
 	if(g_strcmp0(sms_msg->type, "status") != 0) {
-		vvm_error("This is not a status sms!");
+		g_critical("This is not a status sms!");
 		return;
 	}
 	DBG("Previous Provisioning Status to %d", old_provision_status);
@@ -521,7 +522,7 @@ vvm_process_status_message(struct sms_control_message *sms_msg)
 
 		if (modem->enable_vvm) {
 			vvmd_subscribe_service();
-			DBG("Unsubscribing from your carrier's VVM service");
+			DBG("Subscribing to your carrier's VVM service");
 		}
 	} else if (modem->provision_status == VVM_PROVISION_STATUS_NEW ||
 	 	   modem->provision_status == VVM_PROVISION_STATUS_READY) {
@@ -533,9 +534,51 @@ vvm_process_status_message(struct sms_control_message *sms_msg)
 			mm_sync_vvm_imap_server();
 		}
 	} else if (modem->provision_status == VVM_PROVISION_STATUS_UNKNOWN) {
-		vvm_error("Provisioning Status is unknown! Contact your Carrier");
+	        //VZW reports that the Provision status is unknown if the VVM service is unprovisioned
+		if(g_strcmp0(modem->vvm_type, "vvm3") == 0) {
+			if(modem->state == MM_MODEM_STATE_CONNECTED) {
+				const gchar *const *modem_number_ref;
+				const char *country_code;
+				g_autofree char *stripped_number = NULL;
+
+				country_code = get_country_iso_for_mcc (modem->imsi);
+				if (g_strcmp0 (country_code, "US") != 0) {
+					g_warning("you are using a standard for VZW USA but the country code is not US");
+					return;
+				}
+
+				modem_number_ref = mm_modem_get_own_numbers (modem->modem);
+				if (modem_number_ref == NULL) {
+					g_critical("Could not get modem number!");
+					return;
+				}
+				g_autofree char *formatted_number = NULL;
+				formatted_number = phone_utils_format_number_e164(*modem_number_ref,
+								country_code,
+								FALSE);
+				if (formatted_number) {
+			        	char *stripped = NULL;
+					if (g_str_has_prefix (formatted_number, "+1"))
+						stripped = formatted_number + strlen ("+1");
+					else
+						stripped = formatted_number;
+
+					stripped_number = g_strdup(stripped);
+				} else {
+					g_critical("Could not format modem number!");
+					return;
+				}
+				vvm3_activation (stripped_number, sms_msg->activate_url);
+
+			} else {
+				g_warning("Modem is disconnected, cannot attempt to provision vvm3");
+				return;
+			}
+		} else {
+			g_critical("Provisioning Status is unknown! Contact your Carrier");
+	   	}
 	} else if (modem->provision_status == VVM_PROVISION_STATUS_BLOCKED) {
-		vvm_error("Provisioning Status is blocked! Contact your Carrier");
+		g_critical("Provisioning Status is blocked! Contact your Carrier");
 	}
 }
 
@@ -554,7 +597,7 @@ new_sync_vvm_timeout(gpointer user_data)
 	char *dup_msg = user_data;
 	DBG("Processing VVM sync message.");
 	if(vvm_service_new_vvm(modem->service, dup_msg, NULL, modem->vvm_type) == FALSE) {
-		vvm_error("Error making VVM!.");
+		g_critical("Error making VVM!.");
 		g_timeout_add_seconds(60, retry_sync_vvm_imap_server, NULL);
 	}
 	g_free(dup_msg);
@@ -580,7 +623,7 @@ vvmd_process_sms(MMSms	*sms)
 			sms_msg = g_try_new0(struct sms_control_message, 1);
 
 			if(sms_msg == NULL) {
-				vvm_error("Could not allocate space for modem data!");
+				g_critical("Could not allocate space for modem data!");
 			}
 			DBG("Processing VVM status message.");
 			vvm_util_parse_status_sms_message(message, sms_msg, modem->vvm_type);
@@ -597,14 +640,14 @@ vvmd_process_sms(MMSms	*sms)
 		if(sms_message_type == SMS_MESSAGE_STATUS || sms_message_type == SMS_MESSAGE_SYNC) {
 			path = mm_sms_get_path(sms);
 			if(path) {
-				vvm_error("Deleting SMS!");
+				g_critical("Deleting SMS!");
 				mm_modem_messaging_delete(modem->modem_messaging,
 							  path,
 							  NULL,
 							  (GAsyncReadyCallback)cb_sms_delete_finish,
 							  sms);
 			} else {
-				vvm_error("vvmd cannot delete SMS at this time!");
+				g_critical("vvmd cannot delete SMS at this time!");
 			}
 		}
 		g_free(message);
@@ -633,6 +676,11 @@ vvmd_check_pdu_type(MMSms	*sms)
 	MMSmsState	state;
 	MMSmsPduType	pdu_type;
 
+	if(!modem->enable_vvm) {
+		DBG("VVM is not enabled! Not Processing SMS");
+		return;
+	}
+
 	pdu_type = mm_sms_get_pdu_type(sms);
 	state = mm_sms_get_state(sms);
 	switch(pdu_type) {
@@ -680,7 +728,7 @@ cb_sms_list_new_ready(MMModemMessaging	*modemmessaging,
 	list = mm_modem_messaging_list_finish(modemmessaging, result, &error);
 
 	if(!list) {
-		vvm_error("Couldn't get SMS list - error: %s",
+		g_critical("Couldn't get SMS list - error: %s",
 			  error ? error->message : "unknown");
 	} else {
 	for(l = list; l; l = g_list_next(l)) {
@@ -770,7 +818,7 @@ vvmd_disconnect_from_sms_wap(void)
 {
 	MmGdbusModemMessaging *gdbus_sms;
 	if (modem->modem_messaging == NULL) {
-		vvm_error("SMS WAP Disconnect: There is no modem with messaging capabilities!");
+		g_critical("SMS WAP Disconnect: There is no modem with messaging capabilities!");
 		return;
 	}
 	DBG("Stopping watching SMS WAPs");
@@ -784,7 +832,7 @@ vvmd_connect_to_sms_wap(void)
 {
 	MmGdbusModemMessaging *gdbus_sms;
 	if (modem->modem_messaging == NULL) {
-		vvm_error("SMS WAP Connect: There is no modem with messaging capabilities!");
+		g_critical("SMS WAP Connect: There is no modem with messaging capabilities!");
 		return;
 	}
 	gdbus_sms = MM_GDBUS_MODEM_MESSAGING(modem->modem_messaging);
@@ -804,7 +852,7 @@ vvmd_mm_init_modem(MMObject *obj)
 		return FALSE;
 	}
 
-	modem->object = obj;
+	modem->object = g_object_ref(obj);
 	modem->modem = mm_object_get_modem(MM_OBJECT(obj));
 	modem->path = mm_modem_dup_path(modem->modem);
 
@@ -876,20 +924,20 @@ vvmd_mm_add_object(MMObject *obj)
 						   NULL,
 				 		   &error);
 		if(error != NULL) {
-		        vvm_error ("Error Getting Sim: %s", error->message);
+		        g_critical ("Error Getting Sim: %s", error->message);
 			return;
 		}
 		country_code = get_country_iso_for_mcc (mm_sim_get_imsi(sim));
 		modem_number_ref = mm_modem_get_own_numbers (mm_object_get_modem(MM_OBJECT(obj)));
 		if (modem_number_ref == NULL) {
-			vvm_error("Could not get number!");
+			g_critical("Could not get number!");
 			return;
 		}
 		modem_number_formatted = phone_utils_format_number_e164(*modem_number_ref, country_code, FALSE);
 		DBG("Formatted Number %s", modem_number_formatted);
 		if(g_strcmp0(modem_number_formatted, modem->default_number) != 0) {
 			g_free(modem_number_formatted);
-			vvm_error("This modem does not match default number!");
+			g_critical("This modem does not match default number!");
 			return;
 		}
 		g_free(modem_number_formatted);
@@ -929,11 +977,10 @@ vvmd_mm_get_modems(void)
 		vvmd_mm_add_object(MM_OBJECT(l->data));
 	}
 
-	if(!has_modem) {
+	if(!has_modem)
 		vvmd_mm_state(VVMD_MM_STATE_NO_MODEM);
-	} else if(list) {
+	if(list)
 		g_list_free_full(list, g_object_unref);
-	}
 }
 
 
@@ -943,10 +990,10 @@ cb_object_added(GDBusObjectManager 	*manager,
 		gpointer		 user_data)
 {
 	DBG("%s", __func__);
-	if(mm_object_peek_modem_messaging(MM_OBJECT(object))) {
-		vvm_error("New Object does not have Messaging feature, ignoring....");
+	if(mm_object_peek_modem_messaging(MM_OBJECT(object)))
 		vvmd_mm_add_object(MM_OBJECT(object));
-	}
+	else
+		g_warning("New Object does not have Messaging feature, ignoring....");
 
 
 }
@@ -1038,7 +1085,7 @@ cb_mm_manager_new(GDBusConnection 	*connection,
 		vvmd_mm_get_modems();
 
 	} else {
-		vvm_error("Error connecting to ModemManager: %s\n", error->message);
+		g_critical("Error connecting to ModemManager: %s\n", error->message);
 
 		vvmd_mm_state(VVMD_MM_STATE_NO_MANAGER);
 	}
@@ -1047,8 +1094,6 @@ cb_mm_manager_new(GDBusConnection 	*connection,
 static void
 vvmd_mm_get_modem_state(void)
 {
-	g_autoptr(GError) error = NULL;
-
 	if(!modem->modem) {
 		vvmd_mm_state(VVMD_MM_STATE_NO_MODEM);
 		return;
@@ -1075,8 +1120,6 @@ vvmd_mm_get_modem_state(void)
 				return;
 			case MM_MODEM_STATE_DISABLED:
 				DBG("MM_MODEM_STATE_DISABLED");
-				DBG("Turning on Modem....");
-				mm_modem_set_power_state_sync(modem->modem, MM_MODEM_POWER_STATE_ON, NULL, &error);
 				vvmd_mm_state(VVMD_MM_STATE_MODEM_DISABLED);
 				return;
 			case MM_MODEM_STATE_DISABLING:
@@ -1107,6 +1150,18 @@ vvmd_mm_get_modem_state(void)
 			mailbox_interface = mm_bearer_get_interface(modem_bearer);
 			vvm_service_set_interface(modem->service, mailbox_interface);
 		}
+
+		if(g_strcmp0(modem->vvm_type, "vvm3") == 0) {
+			if (modem->provision_status == VVM_PROVISION_STATUS_NOT_SET ||
+			    modem->provision_status == VVM_PROVISION_STATUS_UNKNOWN) {
+				if(!modem->enable_vvm) {
+					DBG("VVM is not enabled!");
+					return;
+				}
+				vvmd_check_subscription_status();
+				return;
+			}
+		}
 	}
 
 	// Automatically retrieve VVM messages when the modem is connected
@@ -1140,7 +1195,7 @@ vvmd_mm_state(int state)
 				vvmd_modem_unavailable();
 				DBG("Modem vanished, Disabling plugin");
 			} else {
-				vvm_error("Could not connect to modem");
+				DBG("Could not connect to modem");
 			}
 			modem->modem_available = FALSE;
 			modem->modem_ready = FALSE;
@@ -1167,12 +1222,22 @@ vvmd_mm_state(int state)
 			break;
 
 		case VVMD_MM_STATE_NO_MANAGER:
+			if(modem->modem_available) {
+				vvmd_modem_unavailable();
+				DBG("Modem vanished, Disabling plugin");
+			}
 			if(modem->manager_available) {
 				g_clear_object(&modem->mm);
+				if(modem->device_arr) {
+					g_ptr_array_set_size(modem->device_arr, 0);
+					g_ptr_array_unref(modem->device_arr);
+					modem->device_arr = NULL;
+				}
 				modem->modem_available = FALSE;
 				modem->modem_ready = FALSE;
 			} else {
-				vvm_error("Could not connect to ModemManager");
+				g_critical("Could not connect to ModemManager");
+				DBG("Could not connect to ModemManager");
 			}
 			modem->manager_available = FALSE;
 			DBG("VVMD_MM_STATE_NO_MANAGER");
@@ -1229,7 +1294,7 @@ cb_sms_send_finish (MMSms        *sms,
 	fin = mm_sms_send_finish (sms, result, &error);
 
 	if (!fin) {
-		vvm_error ("Couldn't send SMS - error: %s", error ? error->message : "unknown");
+		g_critical ("Couldn't send SMS - error: %s", error ? error->message : "unknown");
 	} else {
 		DBG ("Successfully sent SMS: %s", mm_sms_get_path (sms));
 		const char	*path;
@@ -1241,7 +1306,7 @@ cb_sms_send_finish (MMSms        *sms,
 						  (GAsyncReadyCallback)cb_sms_delete_finish,
 						  sms);
 		} else {
-			vvm_error("vvmd cannot process SMS at this time!");
+			g_critical("vvmd cannot process SMS at this time!");
 		}
 
 
@@ -1290,58 +1355,102 @@ vvmd_mm_create_sms (const gchar *message)
 
 static void vvmd_subscribe_service(void)
 {
-	char *message = NULL;
+	g_autofree char *message = NULL;
+
 	if(modem->provision_status == VVM_PROVISION_STATUS_NOT_SET) {
-		/*
-		 * I have experiementally seen that I can unsubscribe to see
-		 * The status of VVM, then take action based on that
-		 */
 		DBG("Your provisioning status is unknown");
 		DBG("Checking Subscription Status");
 		vvmd_check_subscription_status();
 		return;
+	} else if(g_strcmp0(modem->vvm_type, "vvm3") == 0) {
+		DBG("Checking vvm3 subscription status");
+		vvmd_check_subscription_status();
+		return;
 	} else {
 		DBG("You are not be subscribed to your carrier's VVM service");
 	}
+
+	if (modem->modem_available == FALSE) {
+		DBG("Modem not available");
+		return;
+	}
+
+	if (modem->modem_ready == FALSE) {
+		DBG("Modem not ready");
+		return;
+	}
+
 	message = vvm_util_create_activate_sms(modem->carrier_prefix, modem->vvm_type);
 
 	if (message) {
 		DBG("Message: %s", message);
 		vvmd_mm_create_sms (message);
 	}
-	g_free(message);
 }
 
 static void vvmd_unsubscribe_service(void)
 {
-	char *message = NULL;
+	g_autofree char *message = NULL;
+
 	DBG("Unsubscribing from your carrier's VVM service");
 	if (modem->provision_status == VVM_PROVISION_STATUS_NEW ||
-		 	   modem->provision_status == VVM_PROVISION_STATUS_READY) {
+	    modem->provision_status == VVM_PROVISION_STATUS_READY) {
 		if(modem->plugin_registered == TRUE) {
 			vvm_service_deactivate_vvm_imap_server(modem->service);
 		}
 	}
+
+	if (modem->modem_available == FALSE) {
+		DBG("Modem not available");
+		return;
+	}
+
+	if (modem->modem_ready == FALSE) {
+		DBG("Modem not ready");
+		return;
+	}
+
 	message = vvm_util_create_deactivate_sms(modem->carrier_prefix, modem->vvm_type);
 
 	if (message) {
 		DBG("Message: %s", message);
 		vvmd_mm_create_sms (message);
 	}
-	g_free(message);
 }
 
 static void vvmd_check_subscription_status(void)
 {
-	char *message = NULL;
+	g_autofree char *message = NULL;
 	DBG("Checking your carrier's VVM service");
+	if(g_strcmp0(modem->vvm_type, "vvm3") == 0) {
+	        if (modem->provision_status == VVM_PROVISION_STATUS_NOT_SET ||
+		    modem->provision_status == VVM_PROVISION_STATUS_UNKNOWN) {
+			if (modem->state == MM_MODEM_STATE_CONNECTED) {
+				// Give the connection a second so it is usable
+				sleep(1);
+			} else {
+				DBG("Modem is disconnected, cannot attempt to provision vvm3");
+				return;
+			}
+		}
+	}
+
+	if (modem->modem_available == FALSE) {
+		DBG("Modem not available");
+		return;
+	}
+
+	if (modem->modem_ready == FALSE) {
+		DBG("Modem not ready");
+		return;
+	}
+
 	message = vvm_util_create_status_sms(modem->carrier_prefix, modem->vvm_type);
 
 	if (message) {
 		DBG("Message: %s", message);
 		vvmd_mm_create_sms (message);
 	}
-	g_free(message);
 }
 
 static void
@@ -1356,8 +1465,8 @@ find_settings_cb(const char   *carrier,
 	g_autofree char *carrier_prefix = NULL;
 	GDBusConnection	*connection = vvm_dbus_get_connection ();
 	if (error) {
-		vvm_warn("Could not find settings: %s", error->message);
-		vvm_warn("Your VVM settings for are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
+		g_warning("Could not find settings: %s", error->message);
+		g_warning("Your VVM settings for are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
 		return;
 	}
 
@@ -1369,7 +1478,7 @@ find_settings_cb(const char   *carrier,
 		modem_number_ref = mm_modem_get_own_numbers (modem->modem);
 		country_code = get_country_iso_for_mcc (modem->imsi);
 		if (g_strcmp0 (country_code, "US") != 0) {
-			vvm_warn("you are parsing AT&T USA but the country code is not US");
+			g_warning("you are parsing AT&T USA but the country code is not US");
 			return;
 		} else {
 			vvm_std = g_strdup("AT&TUSAProprietary");
@@ -1380,15 +1489,29 @@ find_settings_cb(const char   *carrier,
 							country_code,
 							FALSE);
 			if (formatted_number) {
-			        char *stripped;
+			        char *stripped = NULL;
 				if (g_str_has_prefix (formatted_number, "+1"))
-    					stripped = formatted_number + strlen ("+1");
+					stripped = formatted_number + strlen ("+1");
+				else
+					stripped = formatted_number;
+
 				carrier_prefix = g_strdup(stripped);
 			} else {
-				vvm_error("Could not format modem number!");
+				g_critical("Could not format modem number!");
+				carrier_prefix = g_strdup(*modem_number_ref);
 			}
 		} else {
-			vvm_error("Could not get modem number!");
+			g_critical("Could not get modem number!");
+			carrier_prefix = g_strdup("invalid");
+		}
+	} else if (g_strcmp0("Free Mobile", carrier) == 0 && g_strcmp0("proprietary", vvm_std_to_copy) == 0) {
+		const char *country_code;
+		country_code = get_country_iso_for_mcc (modem->imsi);
+		if (g_strcmp0 (country_code, "FR") != 0) {
+			g_warning("you are parsing French Free mobile but the country code is not FR");
+			return;
+		} else {
+			vvm_std = g_strdup("FreeMobileProprietary");
 		}
 	} else {
 		carrier_prefix = g_strdup(carrier_prefix_to_copy);
@@ -1522,7 +1645,7 @@ static void vvmd_modem_available(void)
 				DBG("VVM carrier settings already set");
 			}
 		} else {
-		        vvm_error ("Error Getting Sim: %s", error->message);
+		        g_critical ("Error Getting Sim: %s", error->message);
 		}
 
 		modem_number_ref = mm_modem_get_own_numbers (modem->modem);
@@ -1530,7 +1653,7 @@ static void vvmd_modem_available(void)
 			vvm_service_set_own_number(modem->service,
 					           *modem_number_ref);
 		} else {
-			vvm_error("Could not get modem number!");
+			g_critical("Could not get modem number!");
 		}
 
 		vvmd_connect_to_sms_wap();
@@ -1566,7 +1689,7 @@ static void vvmd_modem_available(void)
 		modem->state = mm_modem_get_state(modem->modem);
 		vvmd_mm_get_modem_state();
 	} else {
-		vvm_error("Something very bad happened at vvmd_modem_available()!");
+		g_critical("Something very bad happened at vvmd_modem_available()!");
 	}
 
 }
@@ -1585,11 +1708,7 @@ static void vvmd_modem_unavailable(void)
 	g_clear_pointer (&modem->path, g_free);
 	g_clear_object(&modem->modem);
 	g_clear_object(&modem->modem_messaging);
-	modem->object = NULL;
-	if(modem->device_arr && modem->device_arr->len) {
-		g_ptr_array_set_size(modem->device_arr, 0);
-		g_ptr_array_unref(modem->device_arr);
-	}
+	g_clear_object(&modem->object);
 	modem->modem_available = FALSE;
 	modem->modem_ready = FALSE;
 }
@@ -1604,7 +1723,7 @@ static int modemmanager_init(void)
 	modem = g_try_new0(struct modem_data, 1);
 
 	if(modem == NULL) {
-		vvm_error("Could not allocate space for modem data!");
+		g_critical("Could not allocate space for modem data!");
 		return -ENOMEM;
 	}
 
@@ -1618,7 +1737,7 @@ static int modemmanager_init(void)
 						   "VVMEnabled",
 						   &error);
 	if(error) {
-		vvm_error("Enabling Visual Voicemail was not configured! Setting to FALSE.");
+		g_critical("Enabling Visual Voicemail was not configured! Setting to FALSE.");
 		modem->enable_vvm = FALSE;
 
 		g_key_file_set_boolean(modem->modemsettings, SETTINGS_GROUP_MODEMMANAGER,
@@ -1663,7 +1782,7 @@ static int modemmanager_init(void)
 						 SETTINGS_GROUP_MODEMMANAGER,
 						 "DefaultModemNumber", &error);
 	if(error) {
-		vvm_error("No Default Modem Number was configured! Setting to NULL.");
+		g_critical("No Default Modem Number was configured! Setting to NULL.");
 		modem->default_number = g_strdup("NULL");
 
 		g_key_file_set_string(modem->modemsettings, SETTINGS_GROUP_MODEMMANAGER,
@@ -1720,7 +1839,7 @@ static int modemmanager_init(void)
 								   &error);
 
 	if(error) {
-		vvm_error("Error registering VVMD ModemManager interface: %s\n", error->message);
+		g_critical("Error registering VVMD ModemManager interface: %s\n", error->message);
 		g_clear_error(&error);
 	}
 
@@ -1739,23 +1858,26 @@ static void modemmanager_exit(void)
 {
 	GDBusConnection	*connection = vvm_dbus_get_connection ();
 
-	if(modem->plugin_registered == TRUE) {
-		vvm_service_unregister(modem->service);
-		modem->modemsettings = NULL;
-	}
 	if(modem->modem_available) {
 		vvmd_mm_state(VVMD_MM_STATE_NO_MODEM);
 	}
 	if(modem->manager_available) {
 		vvmd_mm_state(VVMD_MM_STATE_NO_MANAGER);
 	}
+	if(modem->plugin_registered == TRUE) {
+		vvm_service_unregister(modem->service);
+		modem->modemsettings = NULL;
+	}
+	vvm_service_unref(modem->service);
 	g_dbus_connection_unregister_object(connection,
 					    modem->registration_id);
 
+	g_free(modem->registered_imsi);
 	g_free(modem->carrier_prefix);
 	g_free(modem->default_number);
 	g_free(modem->vvm_type);
 	g_free(modem->vvm_destination_number);
+	g_bus_unwatch_name(modem->mm_watch_id);
 	g_free(modem);
 	g_dbus_node_info_unref(introspection_data);
 }
