@@ -55,12 +55,12 @@ char
 static char
 *vvm_util_create_att_usa_status_sms(const char *carrier_prefix)
 {
-	vvm_warn("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
-	vvm_warn("!!!!!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!!!!!!");
-	vvm_warn("!!!!!   AT&T USA uses a non-standard protocol for subscribing and   !!!!!!");
-	vvm_warn("!!!!!   unsubscribing. It may change at anytime and no longer work  !!!!!!");
-	vvm_warn("!!!!!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!!!!!!");
-	vvm_warn("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+	g_warning("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+	g_warning("!!!!!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!!!!!!");
+	g_warning("!!!!!   AT&T USA uses a non-standard protocol for subscribing and   !!!!!!");
+	g_warning("!!!!!   unsubscribing. It may change at anytime and no longer work  !!!!!!");
+	g_warning("!!!!!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!WARNING!!!!!!");
+	g_warning("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
 	return g_strdup_printf("GET?c=ATTV:Pixel 5/11:4.4.0.10183&v=1.0&l=%s&AD", carrier_prefix);
 	// Attributes for AT&T USA:
 	// Pixel 5 => Model Number
@@ -70,6 +70,12 @@ static char
 	// carrier_prefix => put your number with no spaces
 	// 		     (i.e if your number is (999) 555-1123, use 9995551123
 
+}
+static char
+*vvm_util_create_freemobile_fr_status_sms(const char *carrier_prefix)
+{
+	return g_strdup_printf("STATE?");
+
 }
 
 char *vvm_util_create_deactivate_sms(const char *carrier_prefix, const char *vvm_type)
@@ -89,11 +95,13 @@ char *vvm_util_create_deactivate_sms(const char *carrier_prefix, const char *vvm
 		// message = g_strdup("%s%s%s;%s<value>;%s<value>", DEACTIVATE_SMS_PREFIX, PROTOCOL_VERSION_PREFIX, PROTOCOL_VERSION_1_3, CLIENT_TYPE_PREFIX, TERMINAL_DESTINATION_PORT_NUMBER_PREFIX);
 		DBG("I do not know how to unsubscribe from an OMTP VVM service!");
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
-		vvm_error("I do not know how to unsubscribe from AT&T USA!");
+		g_critical("I do not know how to unsubscribe from AT&T USA!");
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		g_critical("I do not know how to unsubscribe from Free Mobile!");
 	} else if (g_strcmp0(vvm_type, "vvm3") == 0) {
-		vvm_error("There is no way to unsubscribe from vvm3!");
+		g_critical("There is no way to unsubscribe from vvm3!");
 	} else {
-		vvm_error("Unknown type of VVM service.");
+		g_critical("Unknown type of VVM service.");
 	}
 	return NULL;
 }
@@ -117,10 +125,12 @@ char *vvm_util_create_activate_sms(const char *carrier_prefix, const char *vvm_t
 		DBG("I do not know how to subscribe to an OMTP VVM service!");
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
 		return vvm_util_create_att_usa_status_sms(carrier_prefix);
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		return vvm_util_create_freemobile_fr_status_sms(carrier_prefix);
 	} else if (g_strcmp0(vvm_type, "vvm3") == 0) {
 		return g_strdup_printf("STATUS");
 	} else {
-		vvm_error("Unknown type of VVM service.");
+		g_critical("Unknown type of VVM service.");
 	}
 	return NULL;
 }
@@ -145,10 +155,12 @@ char *vvm_util_create_status_sms(const char *carrier_prefix, const char *vvm_typ
 		DBG("I do not know how check the status of an OMTP VVM service!");
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
 		return vvm_util_create_att_usa_status_sms(carrier_prefix);
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		return vvm_util_create_freemobile_fr_status_sms(carrier_prefix);
 	} else if (g_strcmp0(vvm_type, "vvm3") == 0) {
 		return g_strdup_printf("STATUS");
 	} else {
-		vvm_error("Unknown type of VVM service.");
+		g_critical("Unknown type of VVM service.");
 	}
 	return NULL;
 }
@@ -181,12 +193,32 @@ vvm_util_parse_att_usa_sms_message_type(const char *message, const char *carrier
 		// AT&T has pretty much identical STATUS and SYNC messages, and the SYNC
 		// Messages aren't helpful, just process all as STATUS SMS
 		return SMS_MESSAGE_STATUS;
+	} else if (g_str_has_prefix (message, "GET?AD=")) {
+		// This is from AWESim
+		return SMS_MESSAGE_STATUS;
+	} else if (g_str_has_prefix (message, "STATE?state=")) {
+		// This is a new user message
+		return SMS_MESSAGE_STATUS;
 	} else {
 		DBG("Not a status or sync SMS message");
 		return SMS_MESSAGE_OTHER;
 	}
 }
 
+
+static int
+vvm_util_parse_freemobile_fr_sms_message_type(const char *message, const char *carrier_prefix)
+{
+	if (g_str_has_prefix (message, "STATE?state=")) {
+		return SMS_MESSAGE_STATUS;
+	} else if (g_str_has_prefix (message, "MBOXUPDATE?m=")) {
+		// This is a new user message
+		return SMS_MESSAGE_SYNC;
+	} else {
+		DBG("Not a status or sync SMS message");
+		return SMS_MESSAGE_OTHER;
+	}
+}
 int
 vvm_util_parse_sms_message_type(const char *message, const char *carrier_prefix, const char *vvm_type)
 {
@@ -197,8 +229,10 @@ vvm_util_parse_sms_message_type(const char *message, const char *carrier_prefix,
 		sms_message_type = vvm_util_parse_otmp_sms_message_type(message, carrier_prefix);
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
 		sms_message_type = vvm_util_parse_att_usa_sms_message_type(message, carrier_prefix);
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		sms_message_type = vvm_util_parse_freemobile_fr_sms_message_type(message, carrier_prefix);
 	} else {
-		vvm_error("Unknown VVM type!");
+		g_critical("Unknown VVM type!");
 		sms_message_type = SMS_MESSAGE_OTHER;
 	}
 	return sms_message_type;
@@ -285,7 +319,15 @@ void
 vvm_util_parse_sync_att_usa_sms_message(const char *message,
 				       struct sms_control_message *sms_msg)
 {
-	vvm_error("AT&T has no sync message!!");
+	g_critical("AT&T has no sync message!!");
+}
+
+void
+vvm_util_parse_sync_freemobile_fr_sms_message(const char *message,
+				       struct sms_control_message *sms_msg)
+{
+	DBG("FreeMobile Sync message contain only the number of new messages.");
+	sms_msg->sync_status_reason = SYNC_SMS_MAILBOX_UPDATE;
 }
 
 void
@@ -302,8 +344,10 @@ vvm_util_parse_sync_sms_message(const char *message,
 		vvm_util_parse_sync_otmp_sms_message(message, sms_msg);
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
 		vvm_util_parse_sync_att_usa_sms_message(message, sms_msg);
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		vvm_util_parse_sync_freemobile_fr_sms_message(message, sms_msg);
 	}  else {
-		vvm_error("Unknown VVM type!");
+		g_critical("Unknown VVM type!");
 	}
 }
 
@@ -336,7 +380,6 @@ vvm_util_parse_status_otmp_sms_message(const char *message,
 			continue;
 		if (strlen(single_setting[1]) < 1)
 			continue;
-;
 		if (g_strcmp0(single_setting[0], "st") == 0) { //Provisioning Status
 			if (g_strcmp0(single_setting[1], "N") == 0) {
 				// N = Subscriber New
@@ -388,7 +431,7 @@ vvm_util_parse_status_otmp_sms_message(const char *message,
 				//7 = VVM mailbox not initialised
 				DBG("Return Code: VVM mailbox not initialised.");
 			} else {
-				vvm_error("Unknown Return Code.");
+				g_critical("Unknown Return Code.");
 			}
 
 		} else if (g_strcmp0(single_setting[0], "srv") == 0) { //Mailbox Hostname
@@ -452,7 +495,7 @@ char
 		};
 		for (int j = 0; j<10; j++) {
 			if(i > 14) {
-				vvm_warn("This string should not be longer than 15!");
+				g_warning("This string should not be longer than 15!");
 				decoded = g_string_append_unichar (decoded, decoded_string[i]);
 			} else {
 				if(decode[j][i] == decoded_string[i]) {
@@ -465,6 +508,64 @@ char
 	return to_return;
 }
 
+void
+vvm_util_parse_status_ios_sms_message(const char *message,
+				       struct sms_control_message *sms_msg)
+{
+	g_autofree gchar **status_parts = NULL;
+	g_autofree gchar **hostname_parts = NULL;
+	g_autofree gchar **other_parts = NULL;
+	int other_parts_length;
+
+	if(g_str_has_prefix (message, "STATE?state=")) {
+		message = message + 6;
+		other_parts = g_strsplit(message, ";", -1);
+		other_parts_length = g_strv_length (other_parts);
+		for (int i = 0; i < other_parts_length; i++) {
+		g_autofree gchar **single_setting = NULL;
+			if (strlen(other_parts[i]) < 1)
+				continue;
+			if (other_parts[i] == NULL)
+				continue;
+			single_setting = g_strsplit(other_parts[i], "=", 2);
+			if (single_setting[1] == NULL)
+				continue;
+			if (strlen(single_setting[1]) < 1)
+				continue;
+			if (g_strcmp0(single_setting[0], "state") == 0) {
+				if (g_strcmp0(single_setting[1], "NewAccount") == 0) {
+					sms_msg->provision_status = VVM_PROVISION_STATUS_NEW;
+					DBG("Provisioning Status: New.");
+				} else if (g_strcmp0(single_setting[1], "Active") == 0) {
+					sms_msg->provision_status = VVM_PROVISION_STATUS_READY;
+					DBG("Provisioning Status: Active.");
+				} else if (g_strcmp0(single_setting[1], "NotAvailable") == 0) {
+					sms_msg->provision_status = VVM_PROVISION_STATUS_BLOCKED;
+					DBG("Provisioning Status: Not Available.");
+				} else {
+					sms_msg->provision_status = VVM_PROVISION_STATUS_NOT_SET;
+				}
+			} else if (g_strcmp0(single_setting[0], "server") == 0) {
+				sms_msg->mailbox_hostname = g_strdup(single_setting[1]);
+			} else if (g_strcmp0(single_setting[0], "pw") == 0) {
+				sms_msg->mailbox_password = g_strdup(single_setting[1]);
+			} else if (g_strcmp0(single_setting[0], "port") == 0) { //Mailbox Port
+				if (g_str_match_string("143", single_setting[1], FALSE)) {
+					sms_msg->mailbox_port = g_strdup("143");
+				} else {
+					sms_msg->mailbox_port = g_strdup(single_setting[1]);
+				}
+			} else if (g_strcmp0(single_setting[0], "name") == 0) {
+				g_autofree gchar **name_parts = NULL;
+				sms_msg->mailbox_username = g_strdup(single_setting[1]);
+			} else {
+				//I have no idea what these settings are
+				DBG("Not procesing setting %s", single_setting[0]);
+			}
+		}
+	}
+}
+
 void
 vvm_util_parse_status_att_usa_sms_message(const char *message,
 				       struct sms_control_message *sms_msg)
@@ -474,48 +575,79 @@ vvm_util_parse_status_att_usa_sms_message(const char *message,
 	g_autofree gchar **other_parts = NULL;
 	int other_parts_length;
 
-	status_parts = g_strsplit(message, "?", 2);
+	if(g_str_has_prefix (message, "STATE?state=")) {
+		g_autofree gchar **name_parts = NULL;
 
-	hostname_parts = g_strsplit(status_parts[0], ":", 2);
-	sms_msg->mailbox_hostname = g_strdup(hostname_parts[0]);
-	DBG("Hostname: %s", sms_msg->mailbox_hostname);
+		vvm_util_parse_status_ios_sms_message(message, sms_msg);
 
-	other_parts = g_strsplit(status_parts[1], "&", -1);
-	other_parts_length = g_strv_length (other_parts);
-	for (int i = 0; i < other_parts_length; i++) {
-	g_autofree gchar **single_setting = NULL;
-		if (strlen(other_parts[i]) < 1)
-			continue;
-		if (other_parts[i] == NULL)
-			continue;
-		single_setting = g_strsplit(other_parts[i], "=", 2);
-		if (single_setting[1] == NULL)
-			continue;
-		if (strlen(single_setting[1]) < 1)
-			continue;
-		if (g_strcmp0(single_setting[0], "m") == 0) { //Mailbox Username
-			sms_msg->mailbox_username = g_strdup(single_setting[1]);
+		name_parts = g_strsplit(sms_msg->mailbox_username, ":", -1);
 
-		} else if (g_strcmp0(single_setting[0], "p") == 0) { //Mailbox Password
-			sms_msg->mailbox_password = decode(single_setting[1]);
+		g_clear_pointer(&sms_msg->mailbox_username, g_free);
+		sms_msg->mailbox_username = g_strdup(name_parts[1]);
+	} else {
+		if(g_str_has_prefix (message, "GET?AD=")) {
+			g_autofree gchar *stripped_message = NULL;
+
+			message = message + 7;
+			stripped_message = g_strdup (message);
+			g_strdelimit (stripped_message, "\"", ' ');
+			g_strstrip (stripped_message);
+
+			status_parts = g_strsplit(stripped_message, "?", 2);
+		} else {
+			status_parts = g_strsplit(message, "?", 2);
+		}
+
+		hostname_parts = g_strsplit(status_parts[0], ":", 2);
+		sms_msg->mailbox_hostname = g_strdup(hostname_parts[0]);
+		DBG("Hostname: %s", sms_msg->mailbox_hostname);
+
+		other_parts = g_strsplit(status_parts[1], "&", -1);
+		other_parts_length = g_strv_length (other_parts);
+		for (int i = 0; i < other_parts_length; i++) {
+		g_autofree gchar **single_setting = NULL;
+			if (strlen(other_parts[i]) < 1)
+				continue;
+			if (other_parts[i] == NULL)
+				continue;
+			single_setting = g_strsplit(other_parts[i], "=", 2);
+			if (single_setting[1] == NULL)
+				continue;
+			if (strlen(single_setting[1]) < 1)
+				continue;
+			if (g_strcmp0(single_setting[0], "m") == 0) { //Mailbox Username
+				sms_msg->mailbox_username = g_strdup(single_setting[1]);
+
+			} else if (g_strcmp0(single_setting[0], "p") == 0) { //Mailbox Password
+				sms_msg->mailbox_password = decode(single_setting[1]);
+
+			} else if (g_strcmp0(single_setting[0], "i") == 0) { //Mailbox Port
+				if (g_str_match_string("143", single_setting[1], FALSE)) {
+					sms_msg->mailbox_port = g_strdup("143");
+				} else {
+					sms_msg->mailbox_port = g_strdup(single_setting[1]);
+				}
+			} else if (g_strcmp0(single_setting[0], "S") == 0) { //Provisioning Status
+				if (g_strcmp0(single_setting[1], "I") == 0) {
+					// R = Subscriber Ready
+					sms_msg->provision_status = VVM_PROVISION_STATUS_READY;
+					DBG("Provisioning Status: Ready.");
+				} else if (g_strcmp0(single_setting[1], "U") == 0) {
+					// U = Subscriber Unknown
+					sms_msg->provision_status = VVM_PROVISION_STATUS_UNKNOWN;
+					DBG("Provisioning Status: Unknown.");
+				} else {
+					sms_msg->provision_status = VVM_PROVISION_STATUS_NOT_SET;
+				}
 
-		} else if (g_strcmp0(single_setting[0], "i") == 0) { //Mailbox Port
-			if (g_str_match_string("143", single_setting[1], FALSE)) {
-				sms_msg->mailbox_port = g_strdup("143");
 			} else {
-				sms_msg->mailbox_port = g_strdup(single_setting[1]);
+				//I have no idea what these settings are
+				DBG("Not procesing setting %s", single_setting[0]);
 			}
 
-		} else {
-			//I have no idea what these settings are
-			DBG("Not procesing setting %s", single_setting[0]);
 		}
 
 	}
-
-	sms_msg->provision_status = VVM_PROVISION_STATUS_READY;
-	DBG("Provisioning Status: Ready.");
-
 	//This is from the AT&T VVM app
 	sms_msg->TUI_password_length = g_strdup("7-15");
 }
@@ -534,8 +666,10 @@ vvm_util_parse_status_sms_message(const char *message,
 		vvm_util_parse_status_otmp_sms_message(message, sms_msg);
 	} else if (g_strcmp0(vvm_type, "AT&TUSAProprietary") == 0) {
 		vvm_util_parse_status_att_usa_sms_message(message, sms_msg);
+	} else if (g_strcmp0(vvm_type, "FreeMobileProprietary") == 0) {
+		vvm_util_parse_status_ios_sms_message(message, sms_msg);
 	} else {
-		vvm_error("Unknown VVM type!");
+		g_critical("Unknown VVM type!");
 	}
 }
 
@@ -593,9 +727,9 @@ vvm_util_decode_vvm_headers (struct voicemail *vvm_msg,
 			vvm_msg->content_type = g_strdup(single_token[1]);
 			//DBG("Content-type: %s", single_token[1]);
 		} else if (strlen(single_token[0]) > 0) {
-			vvm_warn("Cannot process Header:%s:containing:%s", single_token[0], single_token[1]);
+			g_debug("Cannot process Header:%s:containing:%s", single_token[0], single_token[1]);
 		} else {
-			vvm_warn("Cannot process line: %s", tokens[i]);
+			g_debug("Cannot process line: %s", tokens[i]);
 		}
 		g_strfreev (single_token);
 	}
@@ -652,11 +786,11 @@ static char
 		}
 	}
 	if (contenttype == NULL) {
-		vvm_error("Unknown content type.");
+		g_critical("Unknown content type.");
 		return NULL;
 	}
 	if (attributes == NULL) {
-		vvm_error("No attributes. This is an attachment, but not one to save");
+		g_debug("No attributes. This is an attachment, but not one to save");
 		return g_strdup("NONE");
 	}
 	attribute_parse = g_strsplit_set(attributes,";", -1);
@@ -671,7 +805,7 @@ static char
 		}
 	}
 	if (filename == NULL) {
-		vvm_error("Could not find a file name.");
+		g_critical("Could not find a file name.");
 		return g_strdup("NONE");
 	}
 
@@ -704,7 +838,7 @@ static char
 	}
 	string_to_decode = g_string_free(decoded, FALSE);
 	if (string_to_decode == NULL) {
-		vvm_error("Error reconstructing attachment!");
+		g_critical("Error reconstructing attachment!");
 		return NULL;
 	} else {
 		//DBG("decoded message: %s", string_to_decode);
@@ -718,7 +852,7 @@ static char
 	} else if (g_strrstr("text/plain", contentencoding) != NULL) {
 		vvm_store (NULL, string_to_decode, NULL, strlen(string_to_decode), filename, folderpath);
 	} else {
-		vvm_error("Do not know how to decode this attachment!");
+		g_critical("Do not know how to decode this attachment!");
 		return NULL;
 	}
 
@@ -745,14 +879,13 @@ vvm_util_decode_vvm_email_multipart_mixed (struct voicemail *vvm_msg,
 	// "_Part_287_1624553961" divides headers and each attachment
 	tokens = g_strsplit_set(vvm_msg->content_type,";", 2);
 	if (g_str_match_string("boundary=", tokens[1], FALSE) == FALSE) {
-		vvm_error("could not find boundary!");
+		g_critical("could not find boundary!");
 		return FALSE;
 	}
 	boundary = g_strsplit_set(tokens[1],"=", 2);
 	g_strdelimit (boundary[1], "\"", ' ');
 	g_strstrip(boundary[1]);
 	DBG("Boundary %s", boundary[1]);
-	//TODO: Decode the message now that you have the boundary
 	email_attachments = g_strsplit(vvm_msg->contents, boundary[1], -1);
 	//The first part are the headers, skip
 	for (int i = 2; email_attachments[i] != NULL; i++) {
@@ -760,10 +893,10 @@ vvm_util_decode_vvm_email_multipart_mixed (struct voicemail *vvm_msg,
 			g_autofree char *save_file_path = NULL;
 			save_file_path = vvm_util_decode_vvm_single_email_attachment (email_attachments[i], folderpath);
 			if (save_file_path == NULL) {
-				vvm_error("Failed to decode attachment");
+				g_critical("Failed to decode attachment");
 				return FALSE;
 			} else if (g_strcmp0(save_file_path, "NONE") == 0) {
-				vvm_error("Not an attachment to save");
+				g_debug("Not an attachment to save");
 			} else {
 				DBG("Putting attachment in settings");
 				if (vvm_msg->attachments == NULL) {
@@ -795,7 +928,7 @@ vvm_util_decode_vvm_all_email_attachments (struct voicemail *vvm_msg,
 	DBG("Attachment Folder Path: %s", new_dir);
 
 	if(vvm_util_decode_vvm_email_multipart_mixed(vvm_msg, new_dir) == FALSE) {
-		vvm_error("Failed to decode message of content type: %s", vvm_msg->content_type);
+		g_critical("Failed to decode message of content type: %s", vvm_msg->content_type);
 		return FALSE;
 	}
 
